]> git.cryptolib.org Git - avr-crypto-lib.git/blobdiff - md5.c
fixed small bug in MD5
[avr-crypto-lib.git] / md5.c
diff --git a/md5.c b/md5.c
index 766fbb43fa229d982e06516e394dd6308b417794..1c87b53f6e9510e0e3592a7123720dc9e03ac4f2 100644 (file)
--- a/md5.c
+++ b/md5.c
@@ -1,9 +1,28 @@
+/* md5.c */
+/*
+    This file is part of the Crypto-avr-lib/microcrypt-lib.
+    Copyright (C) 2008  Daniel Otte (daniel.otte@rub.de)
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 /* 
* File:       md5.h
* Author:     Daniel Otte
* Date:       31.07.2006
* License: GPL
* Description: Implementation of the MD5 hash algorithm as described in RFC 1321
  * 
  */
 
@@ -13,6 +32,8 @@
  #include <stdint.h>
  #include <string.h>
  
+ #undef DEBUG
 void md5_init(md5_ctx_t *s){
        s->counter = 0;
        s->a[0] = 0x67452301;
@@ -39,21 +60,23 @@ uint32_t md5_I(uint32_t x, uint32_t y, uint32_t z){
 
 typedef uint32_t md5_func_t(uint32_t, uint32_t, uint32_t);
 
-#define ROTL32(x,n) ((x)<<(n) | (x)>>(32-(n)))  
+#define ROTL32(x,n) (((x)<<(n)) | ((x)>>(32-(n))))  
 
 void md5_core(uint32_t* a, uint8_t as, void* block, uint8_t k, uint8_t s, uint8_t i, uint8_t fi){
        uint32_t t;
        md5_func_t* funcs[]={md5_F, md5_G, md5_H, md5_I};
-       char funcc[]={'*', '-', '+', '~'};
        as &= 0x3;
        /* a = b + ((a + F(b,c,d) + X[k] + T[i]) <<< s). */
+#ifdef DEBUG
+       char funcc[]={'*', '-', '+', '~'};
        uart_putstr("\r\n DBG: md5_core [");
        uart_putc(funcc[fi]);
        uart_hexdump(&as, 1); uart_putc(' ');
        uart_hexdump(&k, 1); uart_putc(' ');
        uart_hexdump(&s, 1); uart_putc(' ');
        uart_hexdump(&i, 1); uart_putc(']');
-       t = a[as] + funcs[fi](a[(as+1)&3], a[(as+2)&3], a[(as+3)&3]) + ((uint8_t*)block)[k] + md5_T[i] ;
+#endif 
+       t = a[as] + funcs[fi](a[(as+1)&3], a[(as+2)&3], a[(as+3)&3]) + ((uint32_t*)block)[k] + md5_T[i] ;
        a[as]=a[(as+1)&3] + ROTL32(t, s);
 }
 
@@ -61,11 +84,13 @@ void md5_nextBlock(md5_ctx_t *state, void* block){
        uint32_t        a[4];
        uint8_t         m,n,i=0;
        /* this requires other mixed sboxes */
+#ifdef DEBUG
        uart_putstr("\r\n DBG: md5_nextBlock: block:\r\n");
        uart_hexdump(block, 16);        uart_putstr("\r\n");
        uart_hexdump(block+16, 16);     uart_putstr("\r\n");
        uart_hexdump(block+32, 16);     uart_putstr("\r\n");
        uart_hexdump(block+48, 16);     uart_putstr("\r\n");
+#endif 
        
        a[0]=state->a[0];
        a[1]=state->a[1];
@@ -77,10 +102,7 @@ void md5_nextBlock(md5_ctx_t *state, void* block){
        for(m=0;m<4;++m){
                for(n=0;n<4;++n){
                        md5_core(a, 4-n, block, m*4+n, s1t[n],i++,0);
-/*                     t = a[(4-s)&3] + md5_F(a[(4-s-1)&3],a[(4-s-2)&3],a[(4-s-3)&3]) 
-                               + ((uint8_t*)block)[s+4*k] + md5_T[i++];
-                       a[(4-s)&3] = a[(4-s-1)&3] + ROTL32(t, s*5+7);
-*/             }
+               }
        }
        /* round 2 */
        uint8_t s2t[]={5,9,14,20};
@@ -89,14 +111,6 @@ void md5_nextBlock(md5_ctx_t *state, void* block){
                        md5_core(a, 4-n, block, (1+m*4+n*5)&0xf, s2t[n],i++,1);
                }
        }
-/*     for(k=0;k<4;++k){
-               for(s=0;s<4;++s){
-                       t = a[(4-s)&3] + md5_G(a[(4-s-1)&3],a[(4-s-2)&3],a[(4-s-3)&3])
-                               + ((uint8_t*)block)[((1+s*5) +(k*4))&0xf] + md5_T[i++];
-                       a[(4-s)&3] = a[(4-s-1)&3] + ROTL32(t, s2t[s]);
-               }
-       }
-*/
        /* round 3 */
        uint8_t s3t[]={4,11,16,23};
        for(m=0;m<4;++m){
@@ -104,13 +118,6 @@ void md5_nextBlock(md5_ctx_t *state, void* block){
                        md5_core(a, 4-n, block, (5-m*4+n*3)&0xf, s3t[n],i++,2);
                }
        }
-/*     for(k=0;k<4;++k){
-               for(s=0;s<4;++s){
-                       t = a[(4-s)&3] + md5_H(a[(4-s-1)&3],a[(4-s-2)&3],a[(4-s-3)&3])
-                               + ((uint8_t*)block)[((5+s*3)-(k*4))&0xf] + md5_T[i++];
-                       a[(4-s)&3] = a[(4-s-1)&3] + ROTL32(t, s3t[s]);
-               }
-       } */
        /* round 4 */
        uint8_t s4t[]={6,10,15,21};
        for(m=0;m<4;++m){
@@ -118,13 +125,6 @@ void md5_nextBlock(md5_ctx_t *state, void* block){
                        md5_core(a, 4-n, block, (0-m*4+n*7)&0xf, s4t[n],i++,3);
                }
        }
-/*     for(k=0;k<4;++k){
-               for(s=0;s<4;++s){
-                       t = a[(4-s)&3] + md5_I(a[(4-s-1)&3],a[(4-s-2)&3],a[(4-s-3)&3]) 
-                               + ((uint8_t*)block)[((s*7)-(k*4))&0xf] + md5_T[i++];
-                       a[(4-s)&3] = a[(4-s-1)&3] + ROTL32(t, s4t[s]);
-               }
-       } */
        state->a[0] += a[0];
        state->a[1] += a[1];
        state->a[2] += a[2];
@@ -132,32 +132,32 @@ void md5_nextBlock(md5_ctx_t *state, void* block){
        state->counter++;
 }
 
-void md5_lastBlock(md5_ctx_t *state, void* block, uint16_t length){
+void md5_lastBlock(md5_ctx_t *state, void* block, uint16_t length_b){
        uint16_t l;
        uint8_t b[64];
-       while (length >= 512){
+       while (length_b >= 512){
                md5_nextBlock(state, block);
-               length -= 512;
-               block += 512/8;
+               length_b -= 512;
+               block = ((uint8_t*)block) + 512/8;
        }
        memset(b, 0, 64);
-       memcpy(b, block, length/8);
+       memcpy(b, block, length_b/8);
        /* insert padding one */
-       l=length/8;
-       if(length%8){
+       l=length_b/8;
+       if(length_b%8){
                uint8_t t;
                t = ((uint8_t*)block)[l];
-               t |= (0x80>>(length%8));
+               t |= (0x80>>(length_b%8));
                b[l]=t;
        }else{
                b[l]=0x80;
        }
        /* insert length value */
-       if(l+sizeof(uint64_t) > 512/8){
+       if(l+sizeof(uint64_t) >= 512/8){
                md5_nextBlock(state, b);
                state->counter--;
                memset(b, 0, 64);
        }
-       *((uint64_t*)&b[64-sizeof(uint64_t)]) = (state->counter * 512) + length;
+       *((uint64_t*)&b[64-sizeof(uint64_t)]) = (state->counter * 512) + length_b;
        md5_nextBlock(state, b);
 }
@@ -1,9 +1,28 @@
 /* 
- * File:       md5.h
- * Author:     Daniel Otte
- * Date:       31.07.2006
- * License: GPL
- * Description: Implementation of the MD5 hash algorithm as described in RFC 1321
+ * \file       md5.c
+ * \author     Daniel Otte
+ * \date       2006-07-31
+ * \par License:
+ * GPLv3 or later
+ * \brief Implementation of the MD5 hash algorithm as described in RFC 1321
  * 
  */